@@ -14,7 +14,7 @@ import torch.nn.functional as F
 from sklearn import metrics
 from datetime import timedelta
 from sklearn.model_selection import train_test_split
-import pickle
+import json
 from tqdm import tqdm
 import sys
 
@@ -24,7 +24,7 @@ from pytextclassifier.log import logger
 pwd_path = os.path.abspath(os.path.dirname(__file__))
 device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
 MAX_VOCAB_SIZE = 10000  # 词表长度限制
-UNK = '[UNK]',  # 未知字
+UNK = '[UNK]'  # 未知字
 PAD = '[PAD]'  # padding符号
 dropout = 0.5  # 随机失活
 require_improvement = 1000  # 若超过1000batch效果还没提升，则提前结束训练
@@ -39,11 +39,11 @@ n_gram_vocab = 250499  # ngram 词表大小
 tokenizer = lambda x: [y for y in x]  # char-level
 
 
-def load_data(data_filepath, header=None, delimiter='\t', names=['labels', 'text']):
+def load_data(data_filepath, header=None, delimiter='\t', names=('labels', 'text')):
     data_df = pd.read_csv(data_filepath, header=header, delimiter=delimiter, names=names)
     logger.debug(data_df.head())
     X, y = data_df['text'], data_df['labels']
-    return X, y
+    return X, y, data_df
 
 
 def build_vocab(contents, tokenizer, max_size, min_freq):
@@ -64,18 +64,18 @@ def build_vocab(contents, tokenizer, max_size, min_freq):
 
 def build_dataset(X, y, vocab_path, label_vocab_path, pad_size=128):
     if os.path.exists(vocab_path):
-        word_id_map = pickle.load(open(vocab_path, 'rb'))
+        word_id_map = json.load(open(vocab_path, 'r', encoding='utf-8'))
     else:
         word_id_map = build_vocab(X, tokenizer=tokenizer, max_size=MAX_VOCAB_SIZE, min_freq=1)
-        pickle.dump(word_id_map, open(vocab_path, 'wb'))
+        json.dump(word_id_map, open(vocab_path, 'w', encoding='utf-8'), ensure_ascii=False, indent=4)
     logger.debug(f"word vocab size: {len(word_id_map)}, word_vocab_path: {vocab_path}")
 
     if os.path.exists(label_vocab_path):
-        label_id_map = pickle.load(open(label_vocab_path, 'rb'))
+        label_id_map = json.load(open(label_vocab_path, 'r', encoding='utf-8'))
     else:
         id_label_map = {id: v for id, v in enumerate(set(y.tolist()))}
         label_id_map = {v: k for k, v in id_label_map.items()}
-        pickle.dump(label_id_map, open(label_vocab_path, 'wb'))
+        json.dump(label_id_map, open(label_vocab_path, 'w', encoding='utf-8'), ensure_ascii=False, indent=4)
     logger.debug(f"label vocab size: {len(label_id_map)}, label_vocab_path: {label_vocab_path}")
 
     def biGramHash(sequence, t, buckets):
@@ -239,6 +239,7 @@ def evaluate(model, data_iter):
     acc = metrics.accuracy_score(labels_all, predict_all)
     return acc, loss_total / len(data_iter)
 
+
 def train(model, train_iter, dev_iter, num_epochs=10, learning_rate=1e-3, require_improvement=1000, save_path=''):
     # train
     start_time = time.time()
@@ -292,6 +293,14 @@ def load_model(model, model_path):
 
 
 def predict(model, data_list, word_id_map, label_id_map):
+    """
+    Predict data_list
+    @param model: Fasttext model
+    @param data_list: input texts
+    @param word_id_map:
+    @param label_id_map:
+    @return: predict_label, predict_proba
+    """
     model.eval()
 
     def biGramHash(sequence, t, buckets):
@@ -341,7 +350,8 @@ def predict(model, data_list, word_id_map, label_id_map):
             pred = torch.max(outputs, 1)[1].detach().cpu().numpy()
             predict_all = np.append(predict_all, pred)
             log_proba = torch.max(outputs, 1)[0].detach().cpu().numpy()
-            proba = np.exp(-log_proba)
+            proba = 1 - np.exp(-log_proba)
+            # print(f'outputs: {outputs} log_p: {log_proba} p:{proba}')
             proba_all = np.append(proba_all, proba)
     id_label_map = {v: k for k, v in label_id_map.items()}
     predict_label = [id_label_map.get(i) for i in predict_all]
@@ -370,12 +380,12 @@ if __name__ == '__main__':
     torch.cuda.manual_seed_all(SEED)  # 保持结果一致
     print(f'device: {device}')
     # load data
-    X, y = load_data(args.data_path)
+    X, y, _ = load_data(args.data_path)
     print(f'loaded data list, X size: {len(X)}, y size: {len(y)}')
     assert len(X) == len(y)
     print(f'num_classes:{len(set(y))}')
-    word_vocab_path = os.path.join(model_dir, 'word_vocab.pkl')
-    label_vocab_path = os.path.join(model_dir, 'label_vocab.pkl')
+    word_vocab_path = os.path.join(model_dir, 'word_vocab.json')
+    label_vocab_path = os.path.join(model_dir, 'label_vocab.json')
     save_model_path = os.path.join(model_dir, 'model.pth')
     data, word_id_map, label_id_map = build_dataset(X, y, word_vocab_path, label_vocab_path, pad_size)
     train_data, dev_data = train_test_split(data, test_size=0.1, random_state=SEED)
@@ -394,7 +404,10 @@ if __name__ == '__main__':
     for text, label, proba in zip(X[:10], predict_label, predict_proba):
         print(text, label, proba)
     # load new model and predict
+    model = FastTextModel(vocab_size, num_classes).to(device)
+    init_network(model)
     new_model = load_model(model, save_model_path)
-    predict_label, predict_prob = predict(new_model, X[:10], word_id_map, label_id_map)
+    print('new model loaded from file, and predict')
+    predict_label, predict_proba = predict(new_model, X[:10], word_id_map, label_id_map)
     for text, label, proba in zip(X[:10], predict_label, predict_proba):
         print(text, label, proba)
